
/********************************************************
*                                                       *
*       CGI C++ Library v0.23a                          *
*                                                       *
*       (c) 2005 by Ighor Poteryakhin (ighor@ighor.ru)  *
*                                                       *
*       document.cpp                                    *
*                                                       *
********************************************************/

#include "cgi.h"

using namespace std;

namespace cgi{

// class document

    document::document()
    {
	setcp(CP1251);
    }

    document::document(codepage cp)
    {
	setcp(cp);
    }

    void document::setcp(codepage cp)
    {
	_printed=false;
	string s;
	clear();
	_cp=cp;
	s="\t<meta http-equiv=\"content-type\" content=\"text/html;";
	s+=scp();
	s+="\">\n";
	head(s.c_str());
    }

    document::~document()
    {
	// nothing to do...
    }
    
    void document::clear()
    {
	_title="";
	_background="";
	_header.clear();
	_head.clear();
	_body.clear();
    }

    void document::br()
    {
	body("<br>\n");
    }

    void document::title(const char *s)
    {
	if(s!=NULL)
	{
	    string t(s);
	    title(t);
	}
	else
	    title(string(""));
	
    }
    
    void document::title(string s)
    {
	_title=s;
    }

    void document::background(const char *s)
    {
	if(s!=NULL)
	{
	    string t(s);
	    background(t);
	}
	else
	    background(string(""));
	
    }
    
    void document::background(string s)
    {
	_background=s;
    }
    
    void document::body(const char* s)
    {
	if(s!=NULL)
	    _body.push_back(s);
	else
	    _body.push_back("NULL");
    }

    void document::body(string s)
    {
	body(s.c_str());
    }

    void document::body(int s)
    {
	body(itoa(s));
    }

    void document::body(const char* s, halignment align)
    {
	char *salign;
	string p="<p align=\"";
	switch(align)
	{
	    case HALEFT:
		salign="left";
		break;
	    case HARIGHT:
		salign="right";
		break;
	    case HACENTER:
		salign="center";
		break;
	    case HAJUSTIFY:
		salign="justify";
		break;
	}
	p+=salign;
	p+="\">";
	body(p);
	body(s);
	body("</p>");
    }
    
    void document::hr()
    {
	body("<hr>");
    }

    void document::head(const char* s)
    {
	if(s!=NULL)
	    _head.push_back(s);
	else
	    _head.push_back("NULL");
    }

    void document::head(string s)
    {
	head(s.c_str());
    }

    void document::header(const char* s)
    {
	if(s!=NULL && s[0]!='\0')
	{
	    char *ss=(char*)malloc(strlen(s)+1);
	    strcpy(ss,s);
	    // erase trailing ^M & ^J
	    while(ss[strlen(ss)-1]=='\n'||ss[strlen(ss)-1]=='\r')
		ss[strlen(ss)-1]='\0';
	    _header.push_back(ss);
	    free(ss);
	}
	else
	    throw "Can't add empty header";
    }

    void document::header(string s)
    {
	header(s.c_str());
    }

    void document::formstart(char* action, char *method)
    {
	body("<form action=\"");
	body(action);
	body("\" method=");
	body(method);
	body(">\n");
    }
    void document::forminput(char* type, char* name, char* value)
    {
        body("<input type=\"");
	body(type);
	body("\" name=\"");
	body(name);
	body("\" value=\"");
	body(value);
	body("\" >\n");

    }
    void document::formstop()
    {
	body("</form>\n");
    }

    void document::print()
    {
	if(_printed)
	    throw "Already printed!";
	_printed=true;

	text::iterator i;
	for(i=_header.begin();i!=_header.end();i++)
	    cout<<*i<<endl;

        time_t doc_modified;
	time(&doc_modified); // right now
	time_t doc_expires=doc_modified+consts::expire_after;
	cout<<"X-Powered-By: "<<consts::CGI_LIB_VERSION<<endl;
	cout<<"Last-Modified: "<<_gettimem(doc_modified)<<endl;
	cout<<"Expires: "<<_gettimem(doc_expires)<<endl;


	cout<<"Content-type: text/html; charset="<<scp()<<endl;
        cout<<endl;

	cout<<"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">"<<endl;
	cout<<"<html>"<<endl;

	cout<<"<head>"<<endl;
	
	cout<<"\t<title>"<<_title<<"</title>"<<endl;

	for(i=_head.begin();i!=_head.end();i++)
	    cout<<*i;

	cout<<"</head>"<<endl;

	cout<<"<body";
	if(_background.length())
	    cout<<" background=\""<<_background<<"\"";
#ifdef TTM
	cout<<" topmargin=\"0\" leftmargin=\"0\" ";
#endif
#ifdef FOTON
	cout<<" topmargin=\"0\" leftmargin=\"0\" bgcolor=#EFEFEF";
#endif
#ifdef FURGON
	cout<<" topmargin=\"0\" leftmargin=\"0\" style=\"margin:0;\"";
#endif
	cout<<">"<<endl;

	i=_body.begin();
	for(;i!=_body.end();i++)
	    cout<<*i;

	cout<<"</body>"<<endl;

	cout<<"</html>"<<endl;
    }

    codepage document::cp()
    {
	return _cp;
    }
    
    char* document::scp()
    {
	switch(cp())
	{
	    case CP1251:
		return "windows-1251";
	    case CP866:
		return "cp866";
	    case KOI8R:
		return "koi8-r";
	    //...and so on
	}
    }

    void document::body_file(string& filename)
    {
	body_file(filename.c_str());
    }

    void document::body_file(const char* filename)
    {
	ifstream infile(filename);

	if(!infile.good())
	    throw "document::body_file(): Error opening file!";

        try
	{
	    string str;
	    while(getline(infile,str))
	    {
		body(str);
		body("\n");
	    }
	}
	catch(ios_base::failure &var)
	{
	    throw (char*)var.what();
	}
    }

    void document::redirect(string& url)
    {
	redirect(url.c_str());
    }

    void document::redirect(const char* url)
    {
	if(_printed)
	    throw "Page already printed, can't redirect!";
	string header;
	header="Location: ";
	header+=url;
	cout<<header<<endl<<endl;
	exit(1);
    }

}
